package com.bytesw.bytebot.service;

import com.amazonaws.services.s3.model.Bucket;
import com.bytesw.bytebot.http.FileValidationResponse;
import com.bytesw.bytebot.model.BusinessParameter;
import com.bytesw.bytebot.repository.BusinessParameterRepository;
import com.bytesw.xdf.exception.NotFoundException;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3ClientBuilder;
import com.amazonaws.services.s3.model.PutObjectRequest;

import java.io.File;
import java.io.FileOutputStream;
import java.util.List;
import java.util.Optional;

@Service
@Log4j2
public class BucketService {

    @Value("${application.bucket.tenant}")
    private String tenant;

    @Value("${application.bucket.region}")
    private String region;

    @Value("${application.bucket.fileBucket}")
    private String fileDir;

    @Autowired
    private BusinessParameterRepository businessParameterRepository;

    private AmazonS3 getS3Client() {
        return AmazonS3ClientBuilder.standard()
                .withRegion(region)
                .build();
    }

    public boolean generate(byte[] file, FileValidationResponse response) {
        Optional<BusinessParameter> businessParameter = businessParameterRepository.findByKey("Bucket");
        boolean result = false;
        if (!businessParameter.isPresent()) {
            throw new NotFoundException("No existe el parametro de negocio del bucket.");
        }
        String bucketName = businessParameter.get().getDefaultValue();
        File mainFile = new File(response.getFileName());
        try {
            FileOutputStream stream = new FileOutputStream(mainFile);
            stream.write(file);
            String newFile = String.format("%s/%s/1.0/%s/%s",tenant, response.getUuid(), fileDir, mainFile.getName());
            PutObjectRequest request = new PutObjectRequest(bucketName, newFile, mainFile);
            AmazonS3 amazonS3 = getS3Client();
            amazonS3.putObject(request);
            mainFile.delete();
            result = true;
        } catch (Exception e) {
            log.error("Error al cargar archivo");
        }
        return result;
    }

    public void list() {
        AmazonS3 s3 = getS3Client();
        List<Bucket> buckets = s3.listBuckets();
        System.out.println("Your {S3} buckets are:");
        for (Bucket b : buckets) {
            System.out.println("* " + b.getName());
        }
    }

}
